"""
Malte Braack, Naveen Kumar Parameswaran
CAU, Kiel University, Germany

https://www.pec3.org/events/school2022/

Exercise Class 3 - Numerical Optimization
"""

import numpy as np
from numpy import linalg as la
import matplotlib.pyplot as plt
from exercise1 import MyFunction

#------------------------------------------
# Subroutines
#------------------------------------------

def myplot(p):
	plt.figure(figsize=(8, 8))
	plt.plot(p[0], p[1], 'r-o')
	plt.show()

#------------------------------------------

def constantsteplength(f,x,d):
	return 0.1

#------------------------------------------

# Gradient method for minimizing a function f
# using a step size rule 'strategy'
#
class GradientMethod:

	def __init__(self,f,strategy):
		self.f = f
		self.strategy = strategy
		self.eps = 1.e-5
				
	def run(self,x0):
		x = x0
		
		# p is only used for plot output
		p = [ [x[0]] , [x[1]] ]
		kmax = 1000
		
		# kmax = max number of iterations
		for k in range (0,kmax):

			# stopping criterion
			if la.norm ( self.f.grad(x) ) <= self.eps: 
				break		

			# determine decent direction d	
			d = np.array( self.f.grad(x) )
			d *= -1.

			# determine step length alpha	
			alpha = self.strategy(self.f,x,d)

			# update solution x	
			x = np.add(x,alpha*d)

			# store components of x in p for later plot
			p[0].append(x[0])
			p[1].append(x[1])
		print (k, "iterations: ",x)
		myplot(p)
		return x

#------------------------------------------

def Exercise2(x):
	GM = GradientMethod( MyFunction(), constantsteplength)
	GM.run(x)

#------------------------------------------
# Main programme
#------------------------------------------

print ("\nExercise 2:")
x0 =  [0.1 , 1.]
Exercise2(x0)
