"""
Malte Braack, Naveen Kumar Parameswaran
CAU, Kiel University, Germany

https://www.pec3.org/events/school2022/

Exercise Class 3 - Numerical Optimization
"""

import numpy as np
from exercise1 import MyFunction
from exercise2 import GradientMethod
from exercise4 import armijo

#------------------------------------------
# Subroutines
#------------------------------------------

# This class describes a function with the functionalties
# to return the value, the gradient and the Hessian.
# The root of this function states the constraint.
#
class MyEquationConstraint:

	def __init__(self):
		pass

	def value(self,x):
		return x[0]+x[1]-1.1
		
	def grad(self,x):
		return  [1.,1.]
		
	def Hessian(self,x):
		return [ [0,0], [0,0] ]

#------------------------------------------

# This class delivers an augmented function F on basis
# of a given function f and an equation constraint h in
# the form: F(x) = f(x) + alpha*|| h(x) ||^2
#
class AugmentedFunction:

	def __init__(self,f,h,alpha):
		self.alpha = alpha
		self.f = f
		self.h = h
		
	def value(self,x):
		p = self.h.value(x)
		return self.f.value(x) + 0.5*self.alpha*p*p
		
	def grad(self,x):
		dp = np.array(self.h.grad(x))
		dp *= self.alpha*self.h.value(x)
		df = self.f.grad(x)
		y = np.add(df,dp)
		return y
		
	def Hessian(self,x):
		Hf = self.f.Hessian(x)
		Hp1 = self.h.Hessian(x)
		Hp2 = [ self.h.grad(x),self.h.grad(x) ]
		Hp = np.add(Hp1,Hp2)
		Hp *= self.alpha
		return np.add(Hf,Hp)
		
#------------------------------------------

def Exercise5(x0):
	x = x0
	for i in range (0,5):
		alpha = pow(2.,i)
		f = AugmentedFunction(MyFunction(),MyEquationConstraint(),alpha)
		GM = GradientMethod(f,armijo)
		x = GM.run(x)

#------------------------------------------
# Main programme
#------------------------------------------

print ("\nExercise 5:")
x0 =  [0.1 , 1.]
Exercise5(x0)
	
	